@@ -33,7 +33,7 @@
    code, courtesy of Dan Kegel.  */
 
 #include "system.h"
-
+#include "common.h"
 #include <safe-read.h>
 #include <full-write.h>
 
@@ -482,10 +482,7 @@ rmt_open__ (const char *path, int open_mode, int bias, const char *remote_shell)
 	close (from_remote[remote_pipe_number][PREAD]);
 	close (from_remote[remote_pipe_number][PWRITE]);
 
-#if !MSDOS
-	setuid (getuid ());
-	setgid (getgid ());
-#endif
+	sys_reset_uid_gid ();
 
 	if (remote_user)
 	  execl (remote_shell, remote_shell_basename, remote_host,
@@ -608,6 +605,7 @@ rmt_lseek__ (int handle, off_t offset, int whence)
   uintmax_t u = offset < 0 ? - (uintmax_t) offset : (uintmax_t) offset;
   char *p = operand_buffer + sizeof operand_buffer;
 
+  *--p = 0;
   do
     *--p = '0' + (int) (u % 10);
   while ((u /= 10) != 0);
@@ -651,6 +649,7 @@ rmt_ioctl__ (int handle, int operation, char *argument)
 		       : (uintmax_t) ((struct mtop *) argument)->mt_count);
 	char *p = operand_buffer + sizeof operand_buffer;
 	
+        *--p = 0;
 	do
 	  *--p = '0' + (int) (u % 10);
 	while ((u /= 10) != 0);
