@@ -1,7 +1,7 @@
 /* System dependent definitions for GNU tar.
 
-   Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001 Free Software
-   Foundation, Inc.
+   Copyright (C) 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2003
+   Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,23 +22,7 @@
 # include <config.h>
 #endif
 
-/* Declare alloca.  AIX requires this to be the first thing in the file.  */
-
-#if __GNUC__
-# define alloca __builtin_alloca
-#else
-# if HAVE_ALLOCA_H
-#  include <alloca.h>
-# else
-#  ifdef _AIX
- #pragma alloca
-#  else
-#   ifndef alloca
-char *alloca ();
-#   endif
-#  endif
-# endif
-#endif
+#include <alloca.h>
 
 #ifndef __attribute__
 /* This feature is available in gcc versions 2.5 and later.  */
@@ -50,13 +34,6 @@ char *alloca ();
 #include <sys/types.h>
 #include <ctype.h>
 
-#if HAVE_STDDEF_H
-# include <stddef.h>
-#endif
-#ifndef offsetof
-# define offsetof(type, ident) ((size_t) &((type *) 0)->ident)
-#endif
-
 /* IN_CTYPE_DOMAIN (C) is nonzero if the unsigned char C can safely be given
    as an argument to <ctype.h> macros like `isspace'.  */
 #if STDC_HEADERS
@@ -274,10 +251,6 @@ extern int errno;
 #define MODE_RWX	(S_IXUSR | S_IXGRP | S_IXOTH | MODE_RW)
 #define MODE_ALL	(S_ISUID | S_ISGID | S_ISVTX | MODE_RWX)
 
-#ifndef _POSIX_SOURCE
-# include <sys/param.h>
-#endif
-
 /* Include <unistd.h> before any preprocessor test of _POSIX_VERSION.  */
 #if HAVE_UNISTD_H
 # include <unistd.h>
@@ -366,28 +339,17 @@ extern int errno;
    size is greater than 512 bytes; so ST_BLKSIZE code below, in preparation
    for some cleanup in this area, later.  */
 
-/* Get or fake the disk device blocksize.  Usually defined by sys/param.h
-   (if at all).  */
-
-#if !defined(DEV_BSIZE) && defined(BSIZE)
-# define DEV_BSIZE BSIZE
-#endif
-#if !defined(DEV_BSIZE) && defined(BBSIZE) /* SGI */
-# define DEV_BSIZE BBSIZE
-#endif
-#ifndef DEV_BSIZE
-# define DEV_BSIZE 4096
-#endif
-
 /* Extract or fake data from a `struct stat'.  ST_BLKSIZE gives the
    optimal I/O blocksize for the file, in bytes.  Some systems, like
    Sequents, return st_blksize of 0 on pipes.  */
 
+#define DEFAULT_ST_BLKSIZE 512
+
 #if !HAVE_ST_BLKSIZE
-# define ST_BLKSIZE(Statbuf) DEV_BSIZE
+# define ST_BLKSIZE(Statbuf) DEFAULT_ST_BLKSIZE
 #else
 # define ST_BLKSIZE(Statbuf) \
-    ((Statbuf).st_blksize > 0 ? (Statbuf).st_blksize : DEV_BSIZE)
+    ((Statbuf).st_blksize > 0 ? (Statbuf).st_blksize : DEFAULT_ST_BLKSIZE)
 #endif
 
 /* Extract or fake data from a `struct stat'.  ST_NBLOCKS gives the
@@ -458,51 +420,24 @@ extern int errno;
 # include <stdlib.h>
 #else
 void *malloc ();
-void *realloc ();
 char *getenv ();
 #endif
 
-#if HAVE_STDBOOL_H
-# include <stdbool.h>
-#else
-typedef enum {false = 0, true = 1} bool;
-#endif
+#include <stdbool.h>
+#include <stddef.h>
 
 #include <stdio.h>
-
-#ifndef _POSIX_VERSION
-# if MSDOS
-#  include <io.h>
-# else
-off_t lseek ();
-# endif
+#if !defined _POSIX_VERSION && MSDOS
+# include <io.h>
 #endif
 
 #if WITH_DMALLOC
-# undef HAVE_VALLOC
+# undef HAVE_DECL_VALLOC
 # define DMALLOC_FUNC_CHECK
 # include <dmalloc.h>
 #endif
 
-#if HAVE_LIMITS_H
-# include <limits.h>
-#endif
-
-#ifndef CHAR_BIT
-# define CHAR_BIT 8
-#endif
-
-#ifndef CHAR_MAX
-# define CHAR_MAX TYPE_MAXIMUM (char)
-#endif
-
-#ifndef UCHAR_MAX
-# define UCHAR_MAX TYPE_MAXIMUM (unsigned char)
-#endif
-
-#ifndef LONG_MAX
-# define LONG_MAX TYPE_MAXIMUM (long)
-#endif
+#include <limits.h>
 
 #ifndef MB_LEN_MAX
 # define MB_LEN_MAX 1
@@ -533,14 +468,6 @@ off_t lseek ();
 
 /* Prototypes for external functions.  */
 
-#ifndef PARAMS
-# if PROTOTYPES
-#  define PARAMS(Args) Args
-# else
-#  define PARAMS(Args) ()
-# endif
-#endif
-
 #if HAVE_LOCALE_H
 # include <locale.h>
 #endif
@@ -548,40 +475,46 @@ off_t lseek ();
 # define setlocale(Category, Locale) /* empty */
 #endif
 
-#if ENABLE_NLS
-# include <libintl.h>
-# define _(Text) gettext (Text)
-#else
-# undef bindtextdomain
-# define bindtextdomain(Domain, Directory) /* empty */
-# undef textdomain
-# define textdomain(Domain) /* empty */
-# define _(Text) Text
-#endif
-#define N_(Text) Text
-
 #include <time.h>
-#ifndef time
+#if defined(HAVE_SYS_TIME_H) && defined(TIME_WITH_SYS_TIME)
+# include <sys/time.h>
+#endif
+#if ! HAVE_DECL_TIME
 time_t time ();
 #endif
 
+#ifdef HAVE_UTIME_H
+# include <utime.h>
+#endif
+
 /* Library modules.  */
 
 #include <dirname.h>
 #include <error.h>
 #include <savedir.h>
+#include <unlocked-io.h>
 #include <xalloc.h>
 
-#if !HAVE_STRSTR
-char *strstr PARAMS ((const char *, const char *));
-#endif
+#include <gettext.h>
+#define _(msgid) gettext (msgid)
+#define N_(msgid) msgid
 
-#if HAVE_VALLOC
-# ifndef valloc
-void *valloc ();
-# endif
-#else
+#if ! defined valloc && ! HAVE_DECL_VALLOC
 # define valloc(Size) malloc (Size)
 #endif
 
-char *xstrdup PARAMS ((char const *));
+#if MSDOS
+# include <process.h>
+# define SET_BINARY_MODE(arc) setmode(arc, O_BINARY)
+# define ERRNO_IS_EACCES errno == EACCES
+# define mkdir(file, mode) (mkdir) (file)
+#else
+# include <pwd.h>
+# include <grp.h>
+# define SET_BINARY_MODE(arc)
+# define ERRNO_IS_EACCES 0
+#endif
+
+#if XENIX
+# include <sys/inode.h>
+#endif
