@@ -1,5 +1,7 @@
 /* Update a tar archive.
-   Copyright 1988, 92, 94, 96, 97, 99, 2000 Free Software Foundation, Inc.
+
+   Copyright (C) 1988, 1992, 1994, 1996, 1997, 1999, 2000, 2001, 2003
+   Free Software Foundation, Inc.
 
    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -30,8 +32,10 @@ extern union block *current_block;
 
 /* We've hit the end of the old stuff, and its time to start writing new
    stuff to the tape.  This involves seeking back one record and
-   re-writing the current record (which has been changed).  */
-int time_to_start_writing;
+   re-writing the current record (which has been changed).
+   FIXME: Either eliminate it or move it to common.h. 
+*/
+bool time_to_start_writing;
 
 /* Pointer to where we started to write in the first record we write out.
    This is used if we can't backspace the output and have to null out the
@@ -78,7 +82,10 @@ append_file (char *path)
 	    read_fatal_details (path, stat_data.st_size - bytes_left,
 				buffer_size);
 	  if (status == 0)
-	    FATAL_ERROR ((0, 0, _("%s: File shrank by %s bytes"),
+	    FATAL_ERROR ((0, 0,
+			  ngettext ("%s: File shrank by %s byte",
+				    "%s: File shrank by %s bytes",
+				    bytes_left),
 			  quotearg_colon (path),
 			  STRINGIFY_BIGINT (bytes_left, buf)));
 
@@ -103,14 +110,16 @@ update_archive (void)
 
   name_gather ();
   open_archive (ACCESS_UPDATE);
-
+  xheader_write_global ();
+  
   while (!found_end)
     {
-      enum read_header status = read_header ();
+      enum read_header status = read_header (false);
 
       switch (status)
 	{
 	case HEADER_STILL_UNREAD:
+	case HEADER_SUCCESS_EXTENDED:
 	  abort ();
 
 	case HEADER_SUCCESS:
@@ -118,16 +127,17 @@ update_archive (void)
 	    struct name *name;
 
 	    if (subcommand_option == UPDATE_SUBCOMMAND
-		&& (name = name_scan (current_file_name), name))
+		&& (name = name_scan (current_stat_info.file_name)) != NULL)
 	      {
 		struct stat s;
 		enum archive_format unused;
 
-		decode_header (current_header, &current_stat, &unused, 0);
+		decode_header (current_header, &current_stat_info, &unused, 0);
 		chdir_do (name->change_dir);
-		if (deref_stat (dereference_option, current_file_name, &s) == 0
-		    && s.st_mtime <= current_stat.st_mtime)
-		  add_avoided_name (current_file_name);
+		if (deref_stat (dereference_option,
+				current_stat_info.file_name, &s) == 0
+		    && s.st_mtime <= current_stat_info.stat.st_mtime)
+		  add_avoided_name (current_stat_info.file_name);
 	      }
 	    skip_member ();
 	    break;
@@ -159,6 +169,7 @@ update_archive (void)
 	      break;
 
 	    case HEADER_END_OF_FILE:
+	    case HEADER_SUCCESS_EXTENDED:
 	      abort ();
 	    }
 	  break;
@@ -168,13 +179,13 @@ update_archive (void)
     }
 
   reset_eof ();
-  time_to_start_writing = 1;
+  time_to_start_writing = true;
   output_start = current_block->buffer;
 
   {
     char *path;
 
-    while (path = name_from_list (), path)
+    while ((path = name_from_list ()) != NULL)
       {
 	if (excluded_name (path))
 	  continue;
